

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL; 

entity lvov is
Port (
   CLK50       : IN  STD_LOGIC;
   RESET_BTN   : IN  STD_LOGIC;
   
   PS2_CLK     : in  STD_LOGIC;
   PS2_DATA    : in  STD_LOGIC;
   
   SRAM_A      : out    std_logic_vector(17 downto 0);
   SRAM_D      : inout  std_logic_vector(15 downto 0);
   SRAM_WE     : out    std_logic;
   SRAM_OE     : out    std_logic;
   SRAM_CS     : out    std_logic;
   SRAM_LB     : out    std_logic;
   SRAM_UB     : out    std_logic;
   
   SWITCH      : IN  STD_LOGIC_VECTOR(9 downto 0);

   VGA_R       : OUT STD_LOGIC_VECTOR(3 downto 0);
   VGA_G       : OUT STD_LOGIC_VECTOR(3 downto 0);
   VGA_B       : OUT STD_LOGIC_VECTOR(3 downto 0);
   VGA_HSYNC   : OUT STD_LOGIC;
   VGA_VSYNC   : OUT STD_LOGIC );
end lvov;

architecture Behavioral of lvov is

   signal CLK        : std_logic;
   signal RESET      : std_logic := '0';
   signal TICK       : std_logic_vector(3 downto 0) := "0000"; 
   
   signal SRAM_DI    : std_logic_vector(7 downto 0);
   signal SRAM_DO    : std_logic_vector(7 downto 0);

   signal VRAM_CS    : std_logic;
   signal CACHE_SWAP : std_logic;
   signal CACHE_WE   : std_logic;
   signal CACHE_A    : std_logic_vector(5 downto 0);
   signal CACHE_DI   : std_logic_vector(7 downto 0);
   signal CACHE_VA   : std_logic_vector(5 downto 0);
   signal CACHE_VD   : std_logic_vector(7 downto 0);
   signal SCANLINE   : std_logic_vector(7 downto 0);

   signal CACHE_RD   : std_logic;
   signal CACHE_CNT  : std_logic_vector (5 downto 0);
      
   signal KEYB_A     : std_logic_vector(7 downto 0);
   signal KEYB_D     : std_logic_vector(7 downto 0);   
   signal KEYB_A2    : std_logic_vector(3 downto 0);
   signal KEYB_D2    : std_logic_vector(3 downto 0);   
   
   signal COLORS     : std_logic_vector(6 downto 0);
   signal ROM_D      : std_logic_vector(7 downto 0);   
   
   signal ROM_INIT   : std_logic;

   signal CPU_CLK    : std_logic;
   signal CPU_MREQ   : std_logic;
   signal CPU_IORQ   : std_logic;
   signal CPU_RD     : std_logic;
   signal CPU_WR     : std_logic;
   signal CPU_A      : std_logic_vector(15 downto 0);
   signal CPU_DI     : std_logic_vector(7 downto 0);
   signal CPU_DO     : std_logic_vector(7 downto 0);

   signal PPI_C0_DO  : std_logic_vector(7 downto 0);
   signal PPI_C0_I_PA: std_logic_vector(7 downto 0);
   signal PPI_C0_O_PA: std_logic_vector(7 downto 0);
   signal PPI_C0_I_PB: std_logic_vector(7 downto 0);
   signal PPI_C0_O_PB: std_logic_vector(7 downto 0);
   signal PPI_C0_I_PC: std_logic_vector(7 downto 0);
   signal PPI_C0_O_PC: std_logic_vector(7 downto 0);
   signal PPI_C0_CS  : std_logic;

   -- FSM States
   type STATE_TYPE is ( IDLE, RAMREAD1,
                        RAMWRITE1, RAMWRITE2,
                        CACHE1, CACHE2, CACHE3, CACHE4 );
                     
   signal STATE : STATE_TYPE := IDLE;
   
begin

   u_PPI_C0 : entity work.I82C55
   port map(   
      RESET       => not RESET,
      ENA         => '1',
      CLK         => CLK,
      I_ADDR      => CPU_A(1 downto 0),
      I_DATA      => CPU_DO,
      O_DATA      => PPI_C0_DO,
      O_DATA_OE_L => open,
      I_CS_L      => PPI_C0_CS,
      I_RD_L      => CPU_RD,
      I_WR_L      => CPU_WR,
      I_PA        => PPI_C0_I_PA,
      O_PA        => PPI_C0_O_PA,
      I_PB        => PPI_C0_I_PB,
      O_PB        => PPI_C0_O_PB,
      I_PC        => PPI_C0_I_PC,
      O_PC        => PPI_C0_O_PC );

   u_CLOCK : entity work.clock
   port map(
      inclk0      => CLK50,
      c0          => CLK );

   u_CPU : entity work.T80se
	port map(
      RESET_n     => RESET,
      CLK_n       => CLK,
      CLKEN       => CPU_CLK,
      WAIT_n      => '1',
      INT_n       => '1',
      NMI_n       => '1',
      BUSRQ_n     => '1',
      M1_n        => open,
      MREQ_n      => CPU_MREQ,
      IORQ_n      => CPU_IORQ,
      RD_n        => CPU_RD,
      WR_n        => CPU_WR,
      RFSH_n      => open,
      HALT_n      => open,
      BUSAK_n     => open,
      A           => CPU_A,
      DI          => CPU_DI,
      DO          => CPU_DO );
   
   u_VIDEO : entity work.video
   port map(
      CLK         => CLK,
      RESET       => '1',
      CACHE_SWAP  => CACHE_SWAP,
      CACHE_A     => CACHE_VA,
      CACHE_D     => CACHE_VD,
      CURRENT_LINE=> SCANLINE,
      COLORS      => COLORS,
      R           => VGA_R,
      G           => VGA_G,
      B           => VGA_B,
      HSYNC       => VGA_HSYNC,
      VSYNC       => VGA_VSYNC );
      
   u_CACHE : entity work.cache
   port map(
      CLK         => CLK,
      AI          => CACHE_A,
      DI          => CACHE_DI,
      WE          => CACHE_WE,
      AO          => CACHE_VA,
      DO          => CACHE_VD,
      CACHE       => CACHE_SWAP );
      
   u_KEYBOARD : entity work.keyboard
   port map(
      CLK         => CLK,
      RESET       => RESET,
      PS2_CLK     => PS2_CLK,
      PS2_DATA    => PS2_DATA,
      KEYB_A      => KEYB_A,
      KEYB_D      => KEYB_D,
      KEYB_A2     => KEYB_A2,
      KEYB_D2     => KEYB_D2 );

   u_ROM : entity work.rom
   port map(
      clock		   => CLK,
      address		=> CPU_A(13 downto 0),
      q           => ROM_D );
      
   SRAM_LB   <= '0';
   SRAM_UB   <= '1';
   SRAM_D    <= "ZZZZZZZZ" & SRAM_DI;
   SRAM_DO   <= SRAM_D(7 downto 0);
   
   VRAM_CS <= not PPI_C0_O_PC(1);
   COLORS <= not PPI_C0_O_PB(6 downto 0);
      
   CLOCK : process(CLK)
   begin
      if rising_edge(CLK) then
         if RESET_BTN = '0' then
            TICK <= (others => '0');
            RESET <= '0';
         else
            TICK <= TICK + 1;
            CPU_CLK <= '0';
            if TICK = "1111" then  -- Generate 2.16MHz (32.5MHz/15) CPU Clock (2.22MHz (20MHz/9) Original Frequency)
               CPU_CLK <= '1';
               TICK <= (others => '0');
               RESET <= '1';
            end if;
         end if;
      end if;
   end process;    

   FSM : process(CLK)
   begin
      if rising_edge(CLK) then
         if RESET = '0' then
            ROM_INIT <= '1';
            SRAM_WE  <= '1';
            SRAM_OE  <= '1';
            SRAM_CS  <= '1';
            SRAM_DI  <= (others => 'Z');
            CACHE_RD <= '0';
            CACHE_CNT <= "000000";    
            PPI_C0_CS<= '1';
            
         else

            CACHE_WE <= '1';
            if CACHE_SWAP = '1' then
               CACHE_RD <= '1';
               CACHE_CNT <= "000000";    
            end if;
            
            case STATE is
                           
               when IDLE =>

                  if TICK = 1 then
                     PPI_C0_CS<= '1';
                     if CPU_MREQ = '0' then
                        if CPU_RD = '0' then ------------------------------------------------------------------------- Read from Memory
                           if (CPU_A(15) = '1' and CPU_A(14) = '1') or ROM_INIT = '1' then            -- Read from ROM
                              CPU_DI <= ROM_D;
                           else                                                                       -- Read from RAM
                              if CPU_A(15) = '0' and VRAM_CS = '0' then
                                 SRAM_A <= "01" & CPU_A;
                              else
                                 SRAM_A <= "00" & CPU_A;
                              end if;
                              SRAM_OE  <= '0';
                              SRAM_CS  <= '0';                           
                              STATE    <= RAMREAD1;
                           end if;
                        elsif CPU_WR = '0' then ---------------------------------------------------------------------- Write to Memory
                           if CPU_A(15) = '0' and VRAM_CS = '0' then
                              SRAM_A <= "01" & CPU_A;
                           else
                              SRAM_A <= "00" & CPU_A;
                           end if;
                           SRAM_WE  <= '0';
                           SRAM_CS  <= '0';                           
                           SRAM_DI  <= CPU_DO;
                           STATE    <= RAMWRITE1;
                        end if;
                     elsif CPU_IORQ = '0' then 

                        if CPU_RD = '0' then ------------------------------------------------------------------------- Read from I/O-Ports
                        
                           CPU_DI <= (others => '1');

                           if CPU_A(7 downto 0) = X"D1" then
                              CPU_DI <= KEYB_D;
                           elsif CPU_A(7 downto 0) = X"D2" then
                              CPU_DI <= KEYB_D2 & KEYB_A2;
                           end if;

                        elsif CPU_WR = '0' then ---------------------------------------------------------------------- Write to I/O-Ports
   
                           ROM_INIT <= '0';

                           if CPU_A(7 downto 3) = "1100" then
                              PPI_C0_CS<= '0';
                              --VRAM_CS <= CPU_DO(1);
--                           elsif CPU_A(7 downto 0) = X"C1" then
--                              COLORS <= CPU_DO(6 downto 0);
                           elsif CPU_A(7 downto 0) = X"D0" then                                       -- Select Keyboard Matrix Column
                              KEYB_A <= CPU_DO;
                           elsif CPU_A(7 downto 0) = X"D2" then
                              KEYB_A2 <= CPU_DO(3 downto 0);
                           end if;

                        end if;
                     else
                        if CACHE_RD = '1' then
                           SRAM_A <= "0101" & SCANLINE & CACHE_CNT;
                           SRAM_OE  <= '0';
                           SRAM_CS  <= '0';    
                           STATE <= CACHE1;
                        end if;
                     end if;
                  end if;
                                    
               when CACHE1 =>
                  CACHE_A <= CACHE_CNT;
                  CACHE_DI <= SRAM_DO;
                  SRAM_OE  <= '1';
                  SRAM_CS  <= '1';    
                  CACHE_WE <= '0';
                  if CACHE_CNT = "111111" then
                     CACHE_RD <= '0';
                  else
                     CACHE_CNT <= CACHE_CNT + '1';
                  end if;
                  STATE <= IDLE;

               when RAMREAD1 =>
                  SRAM_OE  <= '1';
                  SRAM_CS  <= '1';                           
                  CPU_DI <= SRAM_DO;
                  STATE    <= IDLE;                  

               when RAMWRITE1 =>
                  SRAM_WE  <= '1';
                  SRAM_CS  <= '1';                           
                  STATE    <= RAMWRITE2;                  

               when RAMWRITE2 =>
                  SRAM_DI <= (others => 'Z');
                  STATE    <= IDLE;                  
                  
               when OTHERS =>
                  STATE <= IDLE;
                  
            end case;   
         end if;
      end if;
   end process;    

end Behavioral; 